/*
    toplevel.cpp - the main view of the KDE VideoTune

    Requires the Qt widget libraries, available at no cost at
    http://www.troll.no

    Copyright (C) 1997 Frithjof Brestrich
                       brestrich@kde.org


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "kvidtune.h"

#include <qpixmap.h>
#include <qpushbt.h>
#include <qtimer.h>
#include <qfiledlg.h>
#include <qmsgbox.h>
#include <kiconloader.h>
#include <kmsgbox.h>
#include <kpopmenu.h>

#include "dlginfoxserver.h"
#include "dlginfomonitor.h"
#include "dlginfovideo.h"
#include "dlginfomodeline.h"
#include "dlgoptionsconfig.h"
#include "dlgfilesavewizzardtopw.h"
#include "toplevel.h"
#include "config.h"

/***************************************************************************************/

TopLevel::TopLevel(XF86VidMode *vmode):KTopLevelWidget()
{
    LifeAdjust  = TRUE;
    VidMode     = vmode;

    VML_Akt     = new XF86VidModeLine;
    VidMode->GetModeLine(*VML_Akt);
    VML_Restore = new XF86VidModeLine(*VML_Akt);
    VML_Undo    = new XF86VidModeLine(*VML_Akt);

    setCaption(klocale->translate("Video Configuration for KDE"));

    setupMenuBar();
    setupToolBar();
    setupStatusBar();
    setupMainWidget();

    resizeOnFloating();

    update_modeinfo();
    
    show();
}


TopLevel::~TopLevel()
{
  VidMode->Disconnect(); 
  delete VidMode;
}

void TopLevel::resizeOnFloating()
{
    int w = 0;
    int h = 0;

    if ( menubar->menuBarPos()==KMenuBar::Top )    h += menubar->height(); 
    if ( menubar->menuBarPos()==KMenuBar::Bottom ) h += menubar->height(); 
    if ( toolbar->barPos()==KToolBar::Top )        h += toolbar->height(); 
    if ( toolbar->barPos()==KToolBar::Bottom )     h += toolbar->height(); 
    h += statusbar->height(); 
    h += vidtune->height() + vidtune->y();
      
    if ( toolbar->barPos()==KToolBar::Left )       w += toolbar->width(); 
    if ( toolbar->barPos()==KToolBar::Right )      w += toolbar->width(); 
    w += vidtune->width() + vidtune->x();

    resize(w,h);
    setFixedSize(w,h);
}

void TopLevel::configLoad()
{
    KConfig* cfg = kapp->getConfig();
    cfg->setGroup("misc");
    CfgDisplayWarning = cfg->readNumEntry("displaywarning",TRUE);
    cfg->setGroup("path");
    CfgConfigFile = cfg->readEntry("configfile","/etc/XF86Config");
    CfgWorkingDirectory = cfg->readEntry("workingdirectory",".");
}

void TopLevel::configStore()
{
    KConfig* cfg = kapp->getConfig();
    cfg->setGroup("misc");
    cfg->writeEntry("displaywarning",CfgDisplayWarning);
    cfg->setGroup("path");
    cfg->writeEntry("configfile",CfgConfigFile);
    cfg->writeEntry("workingdirectory",CfgWorkingDirectory);
    cfg->sync();
}

void TopLevel::setupMenuBar()
{
    int     i, imax;
    char    *name;
    char    buf[255];
    QString str;

    menu_file    = new QPopupMenu();
    menu_mode    = new QPopupMenu();
    menu_info    = new QPopupMenu();
    menu_options = new QPopupMenu();
    menu_help    = new QPopupMenu();
    
    menu_file->insertItem(klocale->translate("&Save..."), this, SLOT(filesave()));
    menu_file->insertSeparator(-1);
    menu_file->insertItem(klocale->translate("&Exit"), 
                          KApplication::getKApplication(), SLOT(quit()));

    imax = VidMode->GetModeCount();
    for ( i = 1 ; i <= imax ; i++ ) {
      name=VidMode->GetModeName(i);
      snprintf(buf,sizeof(buf),klocale->translate("Setting %u: "),i);
      str=QString(buf)+name;
      menu_mode->insertItem(str, i );
      free(name);
    }
    connect( menu_mode, SIGNAL( activated( int ) ), this, SLOT( videomode( int ) ) );

    menu_info->insertItem(klocale->translate("&X-Server"),  this, SLOT(infoxserver()));
    menu_info->insertItem(klocale->translate("&Monitor"),   this, SLOT(infomonitor()));
    menu_info->insertItem(klocale->translate("&Videomode"), this, SLOT(infovideomode()));
    menu_info->insertItem(klocale->translate("Mode&line"),  this, SLOT(infomodeline()));

    menu_options->insertItem(klocale->translate("&Configure"), 
                             this, SLOT(optionsconfigure()));

    menu_help->insertItem(klocale->translate("&Using KVidTune"), this, SLOT(help()));
    menu_help->insertSeparator(-1);
    menu_help->insertItem(klocale->translate("&About KVidTune..."), this, SLOT(about()));
    menu_help->insertItem(klocale->translate("&About KDE..."), 
                          KApplication::getKApplication(), SLOT(aboutKDE()));
    menu_help->insertItem(klocale->translate("&About QT..."), 
                          this, SLOT(aboutQt()));
    
    menubar = new KMenuBar(this);
    menubar->insertItem(klocale->translate("&File"), menu_file);
    menubar->insertItem(klocale->translate("&Mode"), menu_mode);
    menubar->insertItem(klocale->translate("&Info"), menu_info);
    menubar->insertItem(klocale->translate("&Options"), menu_options);
    menubar->insertSeparator(-1);
    menubar->insertItem(klocale->translate("&Help"), menu_help);
    setMenu(menubar);
    connect(menubar,SIGNAL(moved(menuPosition)),this,SLOT(resizeOnFloating()));
}
 

void TopLevel::setupToolBar()
{
  KIconLoader IconLoader;
    
  toolbar = new KToolBar(this);
    
//  toolbar->insertButton(IconLoader.loadIcon("exit.xpm"), 0, SIGNAL(clicked()), 
//                   	KApplication::getKApplication(), 
//			SLOT(quit()), TRUE, klocale->translate("Exit"));
//  toolbar->insertSeparator();
 
  toolbar->insertButton(IconLoader.loadIcon("kvidtune-save.xpm"), 0, SIGNAL(clicked()), 
                   	this, SLOT(filesave()), TRUE, klocale->translate("Save configuration"));
  toolbar->insertSeparator();
  toolbar->insertButton(IconLoader.loadIcon("back.xpm"), ID_TOOLBAR_PREV, 
			SIGNAL(clicked()), this, SLOT(prevvideomode()), TRUE, 
			klocale->translate("Previous Video Mode"));
  toolbar->insertButton(IconLoader.loadIcon("forward.xpm"), ID_TOOLBAR_NEXT, 
  			SIGNAL(clicked()), this, SLOT(nextvideomode()), TRUE, 
			klocale->translate("Next Video Mode"));
  int m = VidMode->GetMode();
  toolbar->setItemEnabled( ID_TOOLBAR_NEXT , m!=VidMode->GetModeCount() );
  toolbar->setItemEnabled( ID_TOOLBAR_PREV , m!=1 );
  toolbar->insertSeparator();
  toolbar->insertButton(IconLoader.loadIcon("kvidtune-videomodeinfo.xpm"), 0, SIGNAL(clicked()), 
                   	this, SLOT(infovideomode()), TRUE, klocale->translate("Info about current video mode"));
  toolbar->insertSeparator(-1);
  toolbar->insertButton(IconLoader.loadIcon("kvidtune-info.xpm"), 0, SIGNAL(clicked()), 
                   	this, SLOT(about()), TRUE, klocale->translate("Info"));
  toolbar->insertButton(IconLoader.loadIcon("help.xpm"), 0, SIGNAL(clicked()), 
                   	this, SLOT(help()), TRUE, klocale->translate("Help"));
    
  toolbar->setBarPos(KToolBar::Top);
  addToolBar(toolbar);
  connect(toolbar,SIGNAL(moved(BarPosition)),this,SLOT(resizeOnFloating()));
}

void TopLevel::setupStatusBar()
{
    statusbar = new KStatusBar(this);
    statusbar->insertItem(klocale->translate("No status available"), ID_STATUS_GENERAL);
    statusbar->setInsertOrder(KStatusBar::LeftToRight);
    setStatusBar(statusbar);
}

void TopLevel::setupMainWidget()
{
    QWidget *view = new QWidget(this);
    vidtune = new KVidTune(view, NULL);
    vidtune->move(10,10);
    vidtune->show();

    vidtune->SetLifeAdjust(LifeAdjust);

    connect(vidtune,SIGNAL(VidMoveLeft(int)),     this, SLOT(vidmov_left(int)));
    connect(vidtune,SIGNAL(VidMoveRight(int)),    this, SLOT(vidmov_right(int)));
    connect(vidtune,SIGNAL(VidMoveUp(int)),       this, SLOT(vidmov_up(int)));
    connect(vidtune,SIGNAL(VidMoveDown(int)),     this, SLOT(vidmov_down(int)));
    connect(vidtune,SIGNAL(VidSizeWider(int)),    this, SLOT(vidsiz_hmore(int)));
    connect(vidtune,SIGNAL(VidSizeNarrower(int)), this, SLOT(vidsiz_hless(int)));
    connect(vidtune,SIGNAL(VidSizeTaller(int)),   this, SLOT(vidsiz_vmore(int)));
    connect(vidtune,SIGNAL(VidSizeShorter(int)),  this, SLOT(vidsiz_vless(int)));
    connect(vidtune,SIGNAL(VidLifeAdjust(bool)),  this, SLOT(vidcmd_lifeadj(bool)));
    connect(vidtune,SIGNAL(VidActionTest()),      this, SLOT(vidcmd_test()));
    connect(vidtune,SIGNAL(VidActionUndo()),      this, SLOT(vidcmd_undo()));
    connect(vidtune,SIGNAL(VidActionRestore()),   this, SLOT(vidcmd_restore()));
    connect(vidtune,SIGNAL(VidInterlace(bool)),   this, SLOT(vidflg_interlace(bool)));
    connect(vidtune,SIGNAL(VidDoubleScan(bool)),  this, SLOT(vidflg_doublescan(bool)));
    connect(vidtune,SIGNAL(VidComposite(bool)),   this, SLOT(vidflg_composite(bool)));
    connect(vidtune,SIGNAL(VidPHSync(bool)),      this, SLOT(vidflg_phsync(bool)));
    connect(vidtune,SIGNAL(VidNHSync(bool)),      this, SLOT(vidflg_nhsync(bool)));
    connect(vidtune,SIGNAL(VidPVSync(bool)),      this, SLOT(vidflg_pvsync(bool)));
    connect(vidtune,SIGNAL(VidNVSync(bool)),      this, SLOT(vidflg_nvsync(bool)));
    connect(vidtune,SIGNAL(VidPCSync(bool)),      this, SLOT(vidflg_pcsync(bool)));
    connect(vidtune,SIGNAL(VidNCSync(bool)),      this, SLOT(vidflg_ncsync(bool)));

    setView(view); 
}

void TopLevel::ProcessStatus(const char *topic, const char *msg, bool msgbox = TRUE)
{
    if ( msg ) {
      statusbar->changeItem(msg, ID_STATUS_GENERAL);
      if ( msgbox ) 
        KMsgBox::message(this, topic, msg, KMsgBox::STOP, klocale->translate("Close"));
    } else {
      statusbar->changeItem(klocale->translate("Success"), ID_STATUS_GENERAL);
    }
}

void TopLevel::ProcessStatus(const char *topic, bool msgbox = TRUE)
{
    char *et;

    et = VidMode->LastErrorText();

    ProcessStatus(topic,et,msgbox);

    if ( et ) free (et);
}

void TopLevel::update_modeinfo()
{
  char *resinfo;
  QString sresinfo,scold,s;
  char buf[255];

  snprintf(buf,sizeof(buf),"%.1f",VML_Akt->GetVerticalSyncClock());

  resinfo = VidMode->GetModeName(VidMode->GetMode());
  sresinfo = resinfo;
  free(resinfo);
  
  scold.setNum(VidMode->GetColorDepth());
  s = sresinfo + "  x  " + scold + klocale->translate("bit") + " @ " + buf + klocale->translate(" Hz");
  vidtune->SetResInfo((char*)(const char*)s);
  vidtune->SetFlagCheckBoxes(VML_Akt->IsInterlace(),VML_Akt->IsDoublescan(),
                             VML_Akt->IsComposite(),VML_Akt->IsPHSync(),VML_Akt->IsNHSync(),
			     VML_Akt->IsPVSync(),VML_Akt->IsNVSync(),VML_Akt->IsPCSync(),
			     VML_Akt->IsNCSync());
}

void TopLevel::videomode(int mode)
{
  bool b;

  ProcessStatus(klocale->translate("Mode switch failed"),!(b=VidMode->SetMode(mode)));

  if ( b ) {
    VidMode->GetModeLine(*VML_Akt);
    *VML_Undo = *VML_Restore = *VML_Akt;
  }

  int m = VidMode->GetMode();
  toolbar->setItemEnabled( ID_TOOLBAR_NEXT , m!=VidMode->GetModeCount() );
  toolbar->setItemEnabled( ID_TOOLBAR_PREV , m!=1 );
  update_modeinfo();
}

bool TopLevel::vidcmd_apply( bool doit )
{
  bool b = TRUE;
  
  if ( LifeAdjust || doit ) {
    XF86VidModeLine ml;
    if (VidMode->GetModeLine(ml)) {
      // It seems to be possible that "set" works and "get" fails on bad settings
      if (VidMode->SetModeLine(*VML_Akt) && VidMode->GetModeLine(*VML_Akt)) {
        *VML_Undo = ml;
      } else {
        *VML_Akt = ml; // undo faild request
        b = FALSE; 
      }
    } else {
      VML_Akt = VML_Undo; // forced UNDO to last saved and working settings
    }
    ProcessStatus(klocale->translate("Failed to set new settings"),TRUE);
  } else {
  }
/*
  vidtune->SetFlagCheckBoxes(VML_Akt->IsInterlace(), VML_Akt->IsDoublescan(),
                             VML_Akt->IsComposite(), VML_Akt->IsPHSync(),
                             VML_Akt->IsNHSync(), VML_Akt->IsPVSync(),
			     VML_Akt->IsNVSync(), VML_Akt->IsPCSync(),
			     VML_Akt->IsNCSync());
*/
//  printf("Modeline  %s\n",VML_Akt->GetModeline());
  update_modeinfo();
  return b;
}

void TopLevel::filesave()
{
  DlgFileSaveWizzardTopW *dlg;

  dlg = new DlgFileSaveWizzardTopW(this,NULL,VidMode,CfgConfigFile,CfgWorkingDirectory);
  dlg->exec();
  
  delete dlg;
}

void TopLevel::prevvideomode()
{
  videomode(VidMode->GetMode()-1);
}

void TopLevel::nextvideomode()
{
  videomode(VidMode->GetMode()+1);
}

void TopLevel::vidmov_left(int steps)
{
  if (VML_Akt->Move(-steps,0)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to move screen"),
                  klocale->translate("Reached left boarder"),TRUE);
}

void TopLevel::vidmov_right(int steps)
{
  if (VML_Akt->Move(steps,0)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to move screen"),
                  klocale->translate("Reached right boarder"),TRUE);
}

void TopLevel::vidmov_up(int steps)
{
  if (VML_Akt->Move(0,-steps)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to move screen"),
                  klocale->translate("Reached upper boarder"),TRUE);
}

void TopLevel::vidmov_down(int steps)
{
  if (VML_Akt->Move(0,steps)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to move screen"),
                  klocale->translate("Reached lower boarder"),TRUE);
}

void TopLevel::vidsiz_hmore(int steps)
{
  if (VML_Akt->Resize(steps,0)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to resize screen"), 
                  klocale->translate("Requested horizontal values are out of range"),TRUE);
}

void TopLevel::vidsiz_hless(int steps)
{
  if (VML_Akt->Resize(-steps,0)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to resize screen"), 
                  klocale->translate("Requested horizontal values are out of range"),TRUE);
}

void TopLevel::vidsiz_vmore(int steps)
{
  if (VML_Akt->Resize(0,steps)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to resize screen"), 
                  klocale->translate("Requested vertical values are out of range"),TRUE);
}

void TopLevel::vidsiz_vless(int steps)
{
  if (VML_Akt->Resize(0,-steps)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to resize screen"), 
                  klocale->translate("Requested vertical values are out of range"),TRUE);
}

void TopLevel::vidcmd_lifeadj(bool state)
{
   LifeAdjust = state;
   vidcmd_apply();
}

void TopLevel::vidcmd_test()
{
    if (LifeAdjust) return;

    XF86VidModeLine normal;

    if (!VidMode->GetModeLine(normal)) {
      ProcessStatus(klocale->translate("Preparation for test failed."),TRUE);
      return;
    }

    if (VidMode->SetModeLine(*VML_Akt) && VidMode->GetModeLine(*VML_Akt)) {
      QTimer timer;
      KMsgBox msgbox(this, klocale->translate("Test new video mode"),
                     klocale->translate("If you can read this press Apply or Close\n\n"
		     "Old Video mode will be restored in 10 seconds!"),
		     KMsgBox::EXCLAMATION | KMsgBox::DB_THIRD, 
		     klocale->translate("&Apply"),
		     klocale->translate("&Close"),klocale->translate("A&bort"));
      msgbox.setFixedSize(msgbox.size());
      
      connect( &timer, SIGNAL(timeout()), &msgbox, SLOT(b4Pressed()));
      timer.start(10*1000,TRUE);

      switch (msgbox.exec()) {
        /* APPLY=1 */
        case 1:  *VML_Undo = normal;
	         break;
	/* CLOSE=2 */
	case 2:  VidMode->SetModeLine(normal);
	         *VML_Undo = normal;
	         break;
	/* ABORT=3, TIMEOUT=4 */
	default: VidMode->SetModeLine(normal);
	         break;
      }
    } else 
      ProcessStatus(klocale->translate("Failed to set new settings for test"),TRUE);

}

void TopLevel::vidcmd_undo()
{
    if (LifeAdjust) {
      *VML_Akt = *VML_Undo;
      vidcmd_apply();
    } else {
      XF86VidModeLine tmp;
      tmp = *VML_Akt;
      *VML_Akt = *VML_Undo; 
      *VML_Undo = tmp;
/*
      vidtune->SetFlagCheckBoxes(VML_Akt->IsInterlace(), VML_Akt->IsDoublescan(),
                                 VML_Akt->IsComposite(), VML_Akt->IsPHSync(),
                                 VML_Akt->IsNHSync(), VML_Akt->IsPVSync(),
                                 VML_Akt->IsNVSync(), VML_Akt->IsPCSync(),
                                 VML_Akt->IsNCSync());
*/
      update_modeinfo();
    }
}

void TopLevel::vidcmd_restore()
{
    *VML_Undo = *VML_Akt;
    *VML_Akt = *VML_Restore;
    vidcmd_apply();
}

void TopLevel::vidflg_interlace(bool state)
{
  if (VML_Akt->Interlace(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"Interlace\""),TRUE);
}

void TopLevel::vidflg_doublescan(bool state)
{
  if (VML_Akt->Doublescan(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"Doublescan\""),TRUE);
}

void TopLevel::vidflg_composite(bool state)
{
  if (VML_Akt->Composite(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"Composite\""),TRUE);
}

void TopLevel::vidflg_phsync(bool state)
{
  if (VML_Akt->PHSync(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"+HSync\""),TRUE);
}

void TopLevel::vidflg_nhsync(bool state)
{
  if (VML_Akt->NHSync(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"-HSync\""),TRUE);
}

void TopLevel::vidflg_pvsync(bool state)
{
  if (VML_Akt->PVSync(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"+VSync\""),TRUE);
}

void TopLevel::vidflg_nvsync(bool state)
{
  if (VML_Akt->NVSync(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"-VSync\""),TRUE);
}

void TopLevel::vidflg_pcsync(bool state)
{
  if (VML_Akt->PCSync(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"+CSync\""),TRUE);
}

void TopLevel::vidflg_ncsync(bool state)
{
  if (VML_Akt->NCSync(state)) {
    vidcmd_apply();
  } else 
    ProcessStatus(klocale->translate("Failed to set flag"),
                  klocale->translate("Cannot change Option \"-CSync\""),TRUE);
}

void TopLevel::infoxserver()
{
  DlgInfoXServer *dlg = new DlgInfoXServer;
  char *protocol     = VidMode->X11ProtocolVersion();
  char *vendor       = VidMode->X11Vendor();
  char *release      = VidMode->X11ReleaseVersion();
  char *extversion   = VidMode->XF86VidModeVersion();
  
  dlg->FillLabels(protocol,vendor,release,extversion);
  free(protocol);
  free(vendor);
  free(release);
  free(extversion);
  
  dlg->exec();
  
  delete dlg;
}

void TopLevel::infomonitor()
{
  char *vendor    = VidMode->MonitorVendor();
  char *model     = VidMode->MonitorModel();
  char *bandwidth = VidMode->MonitorBandwidth();
  char *hsync     = VidMode->MonitorSyncRatesH();
  char *vsync     = VidMode->MonitorSyncRatesV();
		     
  DlgInfoMonitor *dlg = new DlgInfoMonitor;

  dlg->FillLabels(vendor,model,bandwidth,hsync,vsync);
  free(vendor);
  free(model);
  free(bandwidth);
  free(hsync),
  free(vsync);

  dlg->exec();

  delete dlg;
}

void TopLevel::infovideomode()
{
  DlgInfoVideo *dlg = new DlgInfoVideo;

  dlg->FillLabels(VML_Akt->GetHorizontalResolution(),
                  VML_Akt->GetVerticalResolution(),
		  VML_Akt->GetHorizontalSyncStart(),
		  VML_Akt->GetHorizontalSyncEnd(),
		  VML_Akt->GetHorizontalSyncTotal(),
		  VML_Akt->GetVerticalSyncStart(),
		  VML_Akt->GetVerticalSyncEnd(),
		  VML_Akt->GetVerticalSyncTotal(),
		  VML_Akt->GetHorizontalSyncClock()/1000.0,
		  VML_Akt->GetVerticalSyncClock(),
		  VML_Akt->GetPixelClock(),
		  VML_Akt->GetFlags());

  dlg->exec();
  delete dlg;
}

void TopLevel::infomodeline()
{
  DlgInfoModeLine *dlg = new DlgInfoModeLine;

  char *ml = VML_Akt->GetModeline();
  dlg->FillLabels( QString("Modeline ") + ml );
  free (ml);

  dlg->exec();
  delete dlg;
}

void TopLevel::optionsconfigure()
{
  DlgOptionsConfig *dlg = new DlgOptionsConfig;

  dlg->FillLabels(CfgDisplayWarning,CfgConfigFile);
       
  if (dlg->exec() == 1) {
  
    dlg->ReadLabels(CfgDisplayWarning,CfgConfigFile);

    configStore();

  }

  delete dlg;
}

void TopLevel::about()
{
    QMessageBox::about(this, klocale->translate("About KVidTune"),
                       klocale->translate("KDE X-Server Video Configuration Utility\n\n"
		  			  "Version "VERSION" (alpha)\n\n"
					  "(c) 1997 by Frithjof Brestrich\n"
					  "(brestrich@kde.org)\n"));
}

void TopLevel::aboutQt()
{
    QMessageBox::aboutQt(this, klocale->translate("About Qt"));
}

void TopLevel::help()
{
    kapp->invokeHTMLHelp( "", "" );    
}
